@@ -1,4 +1,4 @@
-/* ppc32le.c
+/* ppc32.c
  * Copyright 1984-2017 Cisco Systems, Inc.
  * 
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -18,7 +18,6 @@
 
 #include <sys/types.h>
 #include <sys/mman.h>
-#include <unistd.h>
 
 /* NB: when sysconf isn't helpful, hardcoding data max cache line size from PowerMac G4.
  * NB: this may cause illegal instruction error on machines with smaller cache line sizes. Also, it
@@ -53,7 +52,7 @@ void S_doflush(uptr start, uptr end) {
   __asm__ __volatile__ ("sync ; isync");
 }
 
-void S_machine_init() {
+void S_machine_init(void) {
 #if defined(__linux__)
   if ((l1_dcache_line_size = sysconf(_SC_LEVEL1_DCACHE_LINESIZE)) <= 0) {
     l1_dcache_line_size = DEFAULT_L1_MAX_CACHE_LINE_SIZE;
