@@ -713,14 +713,28 @@ static ptr s_system(const char *s) {
   INT status;
 #ifdef PTHREADS
   ptr tc = get_thread_context();
+  char *s_arg = NULL;
 #endif
 
 #ifdef PTHREADS
-  if (DISABLECOUNT(tc) == FIX(0)) deactivate_thread(tc);
+  if (DISABLECOUNT(tc) == FIX(0)) {
+    /* copy `s` in case a GC happens */
+    uptr len = strlen(s) + 1;
+    s_arg = malloc(len);
+    if (s_arg == NULL)
+      S_error("system", "malloc failed");
+    memcpy(s_arg, s, len);
+    deactivate_thread(tc);
+    s = s_arg;
+  } else
+    s_arg = NULL;
 #endif
   status = SYSTEM(s);
 #ifdef PTHREADS
-  if (DISABLECOUNT(tc) == FIX(0)) reactivate_thread(tc);
+  if (DISABLECOUNT(tc) == FIX(0)) {
+    reactivate_thread(tc);
+    free(s_arg);
+  }
 #endif
 
   if ((status == -1) && (errno != 0)) {
@@ -735,8 +749,8 @@ static ptr s_system(const char *s) {
 #ifdef WIN32
   return Sinteger(status);
 #else
-  if WIFEXITED(status) return Sinteger(WEXITSTATUS(status));
-  if WIFSIGNALED(status) return Sinteger(-WTERMSIG(status));
+  if (WIFEXITED(status)) return Sinteger(WEXITSTATUS(status));
+  if (WIFSIGNALED(status)) return Sinteger(-WTERMSIG(status));
   S_error("system", "cannot determine subprocess exit status");
   return 0 /* not reached */;
 #endif /* WIN32 */
@@ -2259,6 +2273,50 @@ static void s_iconv_close(uptr cd) {
   ICONV_CLOSE((iconv_t)cd);
 }
 
+#ifdef DISTRUST_ICONV_PROGRESS
+# define ICONV_FROM iconv_fixup
+static size_t iconv_fixup(iconv_t cd, char **src, size_t *srcleft, char **dst, size_t *dstleft) {
+  size_t r;
+  char *orig_src = *src, *orig_dst = *dst;
+  size_t orig_srcleft = *srcleft, orig_dstleft = *dstleft, srcuntried = 0;
+
+  while (1) {
+    r = ICONV((iconv_t)cd, src, srcleft, dst, dstleft);
+    if ((r == (size_t)-1)
+        && (errno == E2BIG)
+        && ((*srcleft < orig_srcleft) || (*dstleft < orig_dstleft))) {
+      /* Avoid a macOS (as of 14.2.1 and 14.3.1) iconv bug in this
+         case, where we don't trust that consumed input characters are
+         reflected in the output pointer. Reverting progress should be
+         ok for a correct iconv, too, since a -1 result means that no
+         irreversible progress was made. */
+      *src = orig_src;
+      *dst = orig_dst;
+      *srcleft = orig_srcleft;
+      *dstleft = orig_dstleft;
+
+      /* We need to make progress, if possible, to satify normal iconv
+         behavior and "io.ss" expectations. Try converting fewer
+         characters. */
+      if (orig_srcleft > sizeof(string_char)) {
+        size_t try_chars = (orig_srcleft / sizeof(string_char)) / 2;
+        srcuntried += orig_srcleft - (try_chars * sizeof(string_char));
+        orig_srcleft = try_chars * sizeof(string_char);
+        *srcleft = orig_srcleft;
+      } else
+        break;
+    } else
+      break;
+  }
+
+  *srcleft += srcuntried;
+
+  return r;
+}
+#else
+# define ICONV_FROM ICONV
+#endif
+
 #define ICONV_BUFSIZ 400
 
 static ptr s_iconv_from_string(uptr cd, ptr in, uptr i, uptr iend, ptr out, uptr o, uptr oend) {
@@ -2284,7 +2342,8 @@ static ptr s_iconv_from_string(uptr cd, ptr in, uptr i, uptr iend, ptr out, uptr
     under Windows, the iconv dll might have been linked against a different C runtime
     and might therefore set a different errno */
   errno = 0;
-  ICONV((iconv_t)cd, (ICONV_INBUF_TYPE)&inbuf, &inbytesleft, &outbuf, &outbytesleft);
+  ICONV_FROM((iconv_t)cd, (ICONV_INBUF_TYPE)&inbuf, &inbytesleft, &outbuf, &outbytesleft);
+
   new_i = i + inmax - inbytesleft / sizeof(string_char);
   new_o = oend - outbytesleft;
   if (new_i != i || new_o != o) return Scons(Sinteger(new_i), Sinteger(new_o));
