@@ -20,4 +20,4 @@
 /* The C linker may require a reference to a function to pull in all
    the common declarations. */
 #include "externs.h"
-void scheme_statics() { }
+void scheme_statics(void) { }
