@@ -41,6 +41,7 @@ static void init_flfxnum(Scheme_Startup_Env *env);
 static void init_extfl(Scheme_Startup_Env *env);
 static void init_futures(Scheme_Startup_Env *env);
 static void init_foreign(Scheme_Startup_Env *env);
+static void init_terminal(Scheme_Startup_Env *env);
 
 static void skip_certain_things(Scheme_Object *o, Scheme_Close_Custodian_Client *f, void *data);
 
@@ -261,6 +262,7 @@ static void init_startup_env(void)
   /* The ordering of the first few init calls is important, so add to
      the end of the list, not the beginning. */
   MZTIMEIT(symbol-type, scheme_init_symbol_type(env));
+  MZTIMEIT(realm, scheme_init_realm());
   MZTIMEIT(fun, scheme_init_fun(env));
   MZTIMEIT(symbol, scheme_init_symbol(env));
   MZTIMEIT(list, scheme_init_list(env));
@@ -315,6 +317,7 @@ static void init_startup_env(void)
   init_flfxnum(env);
   init_extfl(env);
   init_futures(env);
+  init_terminal(env);
 
   builtin_unsafe_start = builtin_ref_counter;
   scheme_init_unsafe_linklet(env);
@@ -354,6 +357,7 @@ static void init_unsafe(Scheme_Startup_Env *env)
   scheme_init_unsafe_fun(env);
   scheme_init_unsafe_thread(env);
   scheme_init_unsafe_port(env);
+  scheme_init_unsafe_error(env);
 
   scheme_init_extfl_unsafe_number(env);
   scheme_init_extfl_unsafe_numarith(env);
@@ -399,6 +403,15 @@ static void init_foreign(Scheme_Startup_Env *env)
   scheme_init_foreign(env);
 }
 
+static void init_terminal(Scheme_Startup_Env *env)
+{
+  scheme_switch_prim_instance(env, "#%terminal");
+
+  scheme_init_terminal(env);
+
+  scheme_restore_prim_instance(env);
+}
+
 /*========================================================================*/
 /*                     place-specific initialization                       */
 /*========================================================================*/
