@@ -914,10 +914,12 @@ int (*scheme_impersonator_of)(Scheme_Object *obj1, Scheme_Object *obj2);
 intptr_t (*scheme_hash_key)(Scheme_Object *o);
 #endif
 intptr_t (*scheme_equal_hash_key)(Scheme_Object *o);
+intptr_t (*scheme_equal_hash_key_recur)(Scheme_Object *o, Scheme_Object *r);
 intptr_t (*scheme_equal_hash_key2)(Scheme_Object *o);
 intptr_t (*scheme_recur_equal_hash_key)(Scheme_Object *o, void *cycle_data);
 intptr_t (*scheme_recur_equal_hash_key2)(Scheme_Object *o, void *cycle_data);
 intptr_t (*scheme_equal_always_hash_key)(Scheme_Object *o);
+intptr_t (*scheme_equal_always_hash_key_recur)(Scheme_Object *o, Scheme_Object *r);
 intptr_t (*scheme_equal_always_hash_key2)(Scheme_Object *o);
 intptr_t (*scheme_eqv_hash_key)(Scheme_Object *o);
 intptr_t (*scheme_eqv_hash_key2)(Scheme_Object *o);
